#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
# Top level wrapper for launcing Molecular Replacement and Refinement jobs
# Ronan Keegan 10/03/06
#

import os, sys, string
import Model_Queue
import MRBUMP_Phaser
import MRBUMP_Molrep


class MR_submit:

   def __init__(self):
      self.name=""
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag

   def MR_start(self, target_info, init, mstat):
      """ A function to submit a molrep job to an SGE queue. """

      sys.stdout.write("--------------------------------------------------------\n")
      sys.stdout.write("Molecular Replacement log message: Launching MR jobs\n")
      sys.stdout.write("--------------------------------------------------------\n")

      # Sort the model list so that jobs with a higher sequence identity are started first

      self.sort_model_list(mstat)
      job_id_log = open(mstat.search_dir + '/logs/jobs.txt','w')
 
      for model in mstat.sorted_model_list:

         MODEL=mstat.model_list[model]
   
         # Set the job number
         MODEL.setMR_JobNumber(mstat.mr_counter)
         
         # Print some details about this job
         self.print_job_details(MODEL, mstat)

         # Initialise some Refmac parameters
         MODEL.setRefmacPDBfile(MODEL.model_directory + "/refine/refmac_" + MODEL.name + ".pdb")
         MODEL.setRefmacMTZINfile(target_info.hklin)
         MODEL.setRefmacMTZOUTfile(MODEL.model_directory + "/refine/refmac_" + MODEL.name + ".mtz")

         # if we are using Phaser as the MR program then we start the MR job accordingly.
         if mstat.MRPROGRAM=='PHASER':

            # Setup for Phaser
            MODEL.setPhaserLogFile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.log')
            MODEL.setPhaserSummary(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.sum')
            MODEL.setPhaserSolnFile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.sol')
            MODEL.setPhaserMTZfile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.1.mtz')
            MODEL.setPhaserPDBfile(MODEL.model_directory + '/mr/phaser_' + MODEL.name + '.1.pdb')
            MODEL.setPhaserJobName("MRB_" + MODEL.name)

            # Create specific files for the Refmac solution file and logfile for Phaser.
            MODEL.setRefmacLogFile(MODEL.model_directory +'/refine/refmac_phaser_' + MODEL.name + '.log')

            # Complete the jobs script to be Phaser specific
            phaser_job=MRBUMP_Phaser.Phaser()
            phaser_job.submit_job(init, mstat, MODEL, target_info)
      
         # or if we are using molrep...
         if mstat.MRPROGRAM=='MOLREP':

            # Setup for Molrep   
            MODEL.setMolrepLogFile(MODEL.model_directory + '/mr/molrep.log')
            MODEL.setMolrepSolnFile(MODEL.model_directory + '/mr/molrep.log')
            MODEL.setMolrepMTZfile(MODEL.model_directory + '/mr/molrep_' + MODEL.name + '.1.mtz')
            MODEL.setMolrepPDBfile(MODEL.model_directory + '/mr/molrep_' + MODEL.name + '.1.pdb')
            MODEL.setMolrepJobName("MRB_" + MODEL.name)

            # Create specific files for the Refmac solution file and logfile for Molrep.
            MODEL.setRefmacLogFile(MODEL.model_directory +'/refine/molrep_refmac_' + MODEL.name + '.log')

            # Complete the jobs script to be Phaser specific
            molrep_job=MRBUMP_Molrep.Molrep()
            molrep_job.submit_job(init, mstat, MODEL, target_info)
      
         # If we are using the cluster we do the following:
         if init.keywords.CLUSTER == False:

            # If we are running on a single machine we do the following. Uses the same script
            sys.stdout.write("Finished job, checking results....\n")

            # Check the results for this job
            queue=Model_Queue.Queue_info()
            queue.check_result_serial(mstat, model)

            # If a solution is found output the results and exit                
            if mstat.solution_found == True:
               mstat.mr_results.copy_solution_files(mstat, mstat.results_dir + '/solution')
               mstat.mr_results.display_results(mstat, target_info)
               break

	 # Increment the counter 
	 mstat.mr_counter=mstat.mr_counter+1
         mstat.model_list[model]=MODEL

      job_id_log.close()

   def print_job_details(self, MODEL, mstat):
      """ A function to output the details of the MR job to be submitted for execution. """

      # Print some details about this job
      sys.stdout.write("\n")
      sys.stdout.write("*******************************************************************************************\n")
      sys.stdout.write("\n")
      sys.stdout.write("#####################################################\n")
      sys.stdout.write("Template Model %s: %s\n" % (MODEL.MR_job_number, MODEL.name))
      sys.stdout.write("#####################################################\n")
      sys.stdout.write("\n")
      sys.stdout.write("Working directory:\n    %s\n" % MODEL.model_directory)

      # List the models being used as input to the MR job
      if MODEL.number_ensem > 1:
         sys.stdout.write("Input search models: \n")
         for i in range(MODEL.number_ensem):
            sys.stdout.write("   " + MODEL.PDBfile[i] + "\n") 
      else:
         sys.stdout.write("Input search model:\n   %s\n" % MODEL.PDBfile[0])

      sys.stdout.write("Search model type:            %s\n" % MODEL.type)
      sys.stdout.write("Sequence identity:            %.3f\n" % MODEL.seqID[0])
      sys.stdout.write("Molecular Replcement Program: %s\n" % mstat.MRPROGRAM)
      sys.stdout.write("Refinement Program:           Refmac5\n")
      sys.stdout.write("\n")
 
   def sort_model_list(self, mstat):
      """ Sort the models in the list according to sequence identity with the target structure. """

      for name in mstat.sorted_MR_list:
         for model in mstat.model_list.keys():
            if mstat.model_list[model].chain_source == name:
              mstat.sorted_model_list.append(model)
